#
# CORE
# Copyright (c)2010-2012 the Boeing Company.
# See the LICENSE file included in this distribution.
#
''' Ryu SDN controller user-defined service.
'''

import os

from core.service import CoreService, addservice
from core.misc.ipaddr import IPv4Prefix, IPv6Prefix

class ryuService(CoreService):
    ''' This is a ryu user-defined service. 
    '''
    # a unique name is required, without spaces
    _name = "ryuService"
    # you can create your own group here
    _group = "SDN"
    # list of other services this service depends on
    _depends = ()
    # per-node directories
    _dirs = ()
    # generated files (without a full path this file goes in the node's dir,
    #  e.g. /tmp/pycore.12345/n1.conf/)
    _configs = ('ryuService.sh', )
    # this controls the starting order vs other enabled services
    _startindex = 50
    # list of startup commands, also may be generated during startup
    _startup = ('sh ryuService.sh',)
    # list of shutdown commands
    _shutdown = ('killall ryu-manager')

    @classmethod
    def generateconfig(cls, node, filename, services):
        ''' Return a string that will be written to filename, or sent to the
            GUI for user customization.
        '''
        app_path = "/usr/local/lib/python2.7/dist-packages/ryu/app"
        cfg = "#!/bin/sh\n"
        cfg += "# auto-generated by ryuService (ryuService.py)\n"
        cfg += '/usr/local/bin/ryu-manager --observe-links %s/ofctl_rest.py %s/rest_topology.py' % (app_path, app_path)

        #for ifc in node.netifs():
        #    cfg += 'echo "Node %s has interface %s"\n' % (node.name, ifc.name)
        #    # here we do something interesting 
        #    cfg += "\n".join(map(cls.subnetentry, ifc.addrlist))
        #    break
        return cfg

    @staticmethod
    def subnetentry(x):
        ''' Generate a subnet declaration block given an IPv4 prefix string
            for inclusion in the config file.
        '''
        if x.find(":") >= 0:
            # this is an IPv6 address
            return ""
        else:
            net = IPv4Prefix(x)
            return 'echo "  network %s"' % (net)

# this line is required to add the above class to the list of available services
addservice(ryuService)

